from textual.screen import Screen
from textual.widgets import Static
import subprocess
import threading

class LazyGitScreen(Screen):
    DEFAULT_CSS = """
    LazyGitScreen {
        layout: vertical;
        color: white;
        text-align: center;
        padding: 2;
        align: center middle;
    }
    """
    
    def __init__(self):
        super().__init__()
        self.process = None

    def compose(self):
        yield Static("LazyGit is running... Close LazyGit to return.", id="lazygit-message")

    def on_mount(self):
        self.run_lazygit()

    def run_lazygit(self):
        try:
            self.process = subprocess.Popen(["lazygit"], text=True)
            threading.Thread(target=self.monitor_lazygit, daemon=True).start()
        except FileNotFoundError:
            self.app.pop_screen()
            self.app.notify("LazyGit not found! Please install it.", severity="error")
        except Exception as e:
            self.app.pop_screen()
            self.app.notify(f"Error launching LazyGit: {str(e)}", severity="error")

    def monitor_lazygit(self):
        if self.process:
            self.process.wait()
            self.app.call_from_thread(self.exit_lazygit_screen)

    def exit_lazygit_screen(self):
        if self.app.screen == self:
            self.app.pop_screen()

    def on_unmount(self):
        if self.process and self.process.poll() is None:
            self.process.terminate()
            
    def on_key(self, event):
        event.prevent_default()
        event.stop()



from textual.widgets import TextArea
from textual.reactive import reactive
from rich.style import Style
from textual.widgets.text_area import TextAreaTheme
import os

my_theme = TextAreaTheme(
    name="EditorTheme",
    base_style=Style(bgcolor="#0C0C0C"),
    cursor_style=Style(color="white", bgcolor="blue"),
    cursor_line_style=Style(bgcolor="#2a2a2a"),
    #gutter_style=Style(color="#90908a", bgcolor="#272822"),
    #selection_style=Style(bgcolor="#44475a"),
    #bracket_matching_style=Style(bgcolor="#3a3d41"),
    selection_style=Style(bgcolor="#3b5070"),
    bracket_matching_style=Style(bgcolor="#264f78"),

    syntax_styles = {
        "text": Style(color="#abb2bf"),
        "error": Style(color="#be5046", bgcolor="#22272e"),
        "comment": Style(color="#5c6370", italic=True),
        "comment.block": Style(color="#5c6370", italic=True),
        "comment.block.documentation": Style(color="#5c6370", italic=True),
        "comment.line": Style(color="#5c6370", italic=True),
        "comment.line.double-slash": Style(color="#5c6370", italic=True),
        "comment.line.number-sign": Style(color="#5c6370", italic=True),
        "comment.line.percentage": Style(color="#5c6370", italic=True),
        "constant": Style(color="#d19a66"),
        "constant.builtin": Style(color="#d19a66", bold=True),
        "constant.builtin.boolean": Style(color="#d19a66", bold=True),
        "constant.character": Style(color="#d19a66"),
        "constant.character.escape": Style(color="#d19a66"),
        "constant.character.format": Style(color="#d19a66"),
        "constant.language": Style(color="#d19a66", bold=True),
        "constant.language.null": Style(color="#d19a66", bold=True),
        "constant.language.undefined": Style(color="#d19a66", bold=True),
        "constant.macro": Style(color="#d19a66"),
        "constant.numeric": Style(color="#d19a66"),
        "constant.numeric.binary": Style(color="#d19a66"),
        "constant.numeric.complex": Style(color="#d19a66"),
        "constant.numeric.decimal": Style(color="#d19a66"),
        "constant.numeric.float": Style(color="#d19a66"),
        "constant.numeric.hex": Style(color="#d19a66"),
        "constant.numeric.integer": Style(color="#d19a66"),
        "constant.numeric.octal": Style(color="#d19a66"),
        "constant.other": Style(color="#d19a66"),
        "constant.regexp": Style(color="#d19a66"),
        "constant.rgb-value": Style(color="#d19a66"),
        "boolean": Style(color="#d19a66", bold=True),
        "number": Style(color="#d19a66"),
        "keyword": Style(color="#e06c75"),
        "keyword.control": Style(color="#e06c75"),
        "keyword.control.conditional": Style(color="#e06c75"),
        "keyword.control.directive": Style(color="#e06c75"),
        "keyword.control.flow": Style(color="#e06c75"),
        "keyword.control.import": Style(color="#e06c75"),
        "keyword.control.return": Style(color="#e06c75"),
        "keyword.control.trycatch": Style(color="#e06c75"),
        "keyword.declaration": Style(color="#e06c75"),
        "keyword.declaration.class": Style(color="#e06c75"),
        "keyword.declaration.function": Style(color="#e06c75"),
        "keyword.declaration.method": Style(color="#e06c75"),
        "keyword.declaration.type": Style(color="#e06c75"),
        "keyword.declaration.var": Style(color="#e06c75"),
        "keyword.operator": Style(color="#e06c75"),
        "keyword.operator.arithmetic": Style(color="#e06c75"),
        "keyword.operator.assignment": Style(color="#e06c75"),
        "keyword.operator.comparison": Style(color="#e06c75"),
        "keyword.operator.logical": Style(color="#e06c75"),
        "keyword.operator.new": Style(color="#e06c75"),
        "keyword.other": Style(color="#e06c75"),
        "conditional": Style(color="#e06c75"),
        "repeat": Style(color="#e06c75"),
        "include": Style(color="#e06c75"),
        "exception": Style(color="#e06c75"),
        "operator": Style(color="#e06c75"),
        "operator.assignment": Style(color="#e06c75"),
        "operator.comparison": Style(color="#e06c75"),
        "operator.arithmetic": Style(color="#e06c75"),
        "operator.logical": Style(color="#e06c75"),
        "operator.bitwise": Style(color="#e06c75"),
        "operator.ternary": Style(color="#e06c75"),
        "punctuation": Style(color="#abb2bf"),
        "punctuation.accessor": Style(color="#abb2bf"),
        "punctuation.bracket": Style(color="#abb2bf"),
        "punctuation.bracket.angle": Style(color="#abb2bf"),
        "punctuation.bracket.curly": Style(color="#abb2bf"),
        "punctuation.bracket.round": Style(color="#abb2bf"),
        "punctuation.bracket.square": Style(color="#abb2bf"),
        "punctuation.colon": Style(color="#abb2bf"),
        "punctuation.comma": Style(color="#abb2bf"),
        "punctuation.decorator": Style(color="#e06c75"),
        "punctuation.delimiter": Style(color="#abb2bf"),
        "punctuation.semi": Style(color="#abb2bf"),
        "punctuation.separator": Style(color="#abb2bf"),
        "punctuation.special": Style(color="#61afef"),
        "punctuation.terminator": Style(color="#abb2bf"),
        "string": Style(color="#98c379"),
        "string.documentation": Style(color="#98c379"),
        "string.escape": Style(color="#61afef"),
        "string.heredoc": Style(color="#98c379"),
        "string.interpolated": Style(color="#98c379"),
        "string.other": Style(color="#98c379"),
        "string.quoted": Style(color="#98c379"),
        "string.quoted.double": Style(color="#98c379"),
        "string.quoted.other": Style(color="#98c379"),
        "string.quoted.single": Style(color="#98c379"),
        "string.quoted.triple": Style(color="#98c379"),
        "string.regexp": Style(color="#98c379"),
        "string.special": Style(color="#98c379"),
        "string.template": Style(color="#98c379"),
        "string.unquoted": Style(color="#98c379"),
        "escape": Style(color="#61afef"),
        "variable": Style(color="#abb2bf"),
        "variable.builtin": Style(color="#61afef", bold=True),
        "variable.declaration": Style(color="#abb2bf"),
        "variable.language": Style(color="#c678dd"),
        "variable.language.self": Style(color="#e06c75", italic=True),
        "variable.language.special": Style(color="#c678dd"),
        "variable.language.super": Style(color="#e06c75", italic=True),
        "variable.language.this": Style(color="#e06c75", italic=True),
        "variable.member": Style(color="#abb2bf"),
        "variable.other": Style(color="#abb2bf"),
        "variable.other.constant": Style(color="#d19a66"),
        "variable.other.enummember": Style(color="#d19a66"),
        "variable.other.readwrite": Style(color="#abb2bf"),
        "variable.parameter": Style(color="#abb2bf"),
        "parameter": Style(color="#abb2bf"),
        "property": Style(color="#61afef"),
        "attribute": Style(color="#d19a66"),
        "field": Style(color="#abb2bf"),
        "self": Style(color="#e06c75", italic=True),
        "this": Style(color="#e06c75", italic=True),
        "function": Style(color="#61afef"),
        "function.builtin": Style(color="#61afef", bold=True),
        "function.call": Style(color="#61afef"),
        "function.declaration": Style(color="#61afef", bold=True),
        "function.macro": Style(color="#61afef"),
        "function.method": Style(color="#61afef"),
        "function.method.call": Style(color="#61afef"),
        "function.method.declaration": Style(color="#61afef", bold=True),
        "function.special": Style(color="#61afef", bold=True),
        "method": Style(color="#61afef"),
        "method.call": Style(color="#61afef"),
        "method.declaration": Style(color="#61afef", bold=True),
        "constructor": Style(color="#61afef", bold=True),
        "decorator": Style(color="#e06c75", italic=True),
        "decorator.builtin": Style(color="#e06c75", italic=True, bold=True),
        "type": Style(color="#c678dd"),
        "type.annotation": Style(color="#c678dd"),
        "type.builtin": Style(color="#c678dd", bold=True),
        "type.declaration": Style(color="#c678dd", bold=True),
        "type.definition": Style(color="#c678dd", bold=True),
        "type.parameter": Style(color="#c678dd"),
        "type.primitive": Style(color="#c678dd", bold=True),
        "class": Style(color="#c678dd", bold=True),
        "class.declaration": Style(color="#c678dd", bold=True),
        "class.builtin": Style(color="#c678dd", bold=True),
        "enum": Style(color="#c678dd"),
        "enum.declaration": Style(color="#c678dd", bold=True),
        "enum.member": Style(color="#d19a66"),
        "interface": Style(color="#c678dd"),
        "interface.declaration": Style(color="#c678dd", bold=True),
        "namespace": Style(color="#c678dd"),
        "module": Style(color="#c678dd"),
        "struct": Style(color="#c678dd"),
        "struct.declaration": Style(color="#c678dd", bold=True),
        "typeParameter": Style(color="#c678dd"),
        "union": Style(color="#c678dd"),
        "tag": Style(color="#e06c75"),
        "tag.attribute": Style(color="#61afef"),
        "tag.attribute.name": Style(color="#61afef"),
        "tag.attribute.value": Style(color="#98c379"),
        "tag.delimiter": Style(color="#abb2bf"),
        "tag.name": Style(color="#e06c75"),
        "tag.builtin": Style(color="#e06c75", bold=True),
        "tag.entity": Style(color="#61afef"),
        "tag.id": Style(color="#e06c75"),
        "tag.class": Style(color="#61afef"),
        "css.property": Style(color="#61afef"),
        "css.selector": Style(color="#e06c75"),
        "css.selector.class": Style(color="#61afef"),
        "css.selector.id": Style(color="#e06c75"),
        "css.selector.tag": Style(color="#e06c75"),
        "css.selector.pseudo-class": Style(color="#e06c75"),
        "css.selector.pseudo-element": Style(color="#e06c75"),
        "css.unit": Style(color="#d19a66"),
        "css.color": Style(color="#d19a66"),
        "markup": Style(color="#abb2bf"),
        "markup.bold": Style(color="#abb2bf", bold=True),
        "markup.heading": Style(color="#d19a66", bold=True),
        "markup.heading.1": Style(color="#d19a66", bold=True),
        "markup.heading.2": Style(color="#d19a66", bold=True),
        "markup.heading.3": Style(color="#d19a66", bold=True),
        "markup.heading.4": Style(color="#d19a66", bold=True),
        "markup.heading.5": Style(color="#d19a66", bold=True),
        "markup.heading.6": Style(color="#d19a66", bold=True),
        "markup.italic": Style(color="#abb2bf", italic=True),
        "markup.list": Style(color="#e06c75"),
        "markup.list.numbered": Style(color="#e06c75"),
        "markup.list.unnumbered": Style(color="#e06c75"),
        "markup.quote": Style(color="#5c6370", italic=True),
        "markup.raw": Style(color="#98c379"),
        "markup.strikethrough": Style(color="#abb2bf", strike=True),
        "markup.underline": Style(color="#abb2bf", underline=True),
        "heading": Style(color="#d19a66", bold=True),
        "link": Style(color="#61afef", underline=True),
        "link_url": Style(color="#98c379", underline=True),
        "emphasis": Style(italic=True),
        "strong": Style(bold=True),
        "list": Style(color="#e06c75"),
        "quote": Style(color="#5c6370", italic=True),
        "label": Style(color="#e06c75"),
        "special": Style(color="#61afef"),
        "source": Style(color="#abb2bf"),
        "meta": Style(color="#abb2bf"),
        "meta.block": Style(color="#abb2bf"),
        "meta.function": Style(color="#abb2bf"),
        "meta.tag": Style(color="#abb2bf"),
        "meta.selector": Style(color="#abb2bf"),
        "diff": Style(color="#abb2bf"),
        "diff.plus": Style(color="#3fb950"),
        "diff.minus": Style(color="#f85149"),
        "diff.delta": Style(color="#d29922"),
        "diff.header": Style(color="#61afef", bold=True),
        "git_commit": Style(color="#abb2bf"),
        "git_rebase": Style(color="#abb2bf"),
        "json.property": Style(color="#61afef"),
        "json.string": Style(color="#98c379"),
        "json.number": Style(color="#d19a66"),
        "json.keyword": Style(color="#e06c75"),
        "yaml.key": Style(color="#61afef"),
        "yaml.value": Style(color="#98c379"),
        "yaml.anchor": Style(color="#c678dd"),
        "shell.builtin": Style(color="#61afef", bold=True),
        "shell.command": Style(color="#61afef"),
        "shell.operator": Style(color="#e06c75"),
        "shell.variable": Style(color="#c678dd"),
        "python.builtin": Style(color="#61afef", bold=True),
        "python.decorator": Style(color="#e06c75", italic=True),
        "python.self": Style(color="#e06c75", italic=True),
        "python.magic": Style(color="#61afef", bold=True),
        "python.fstring": Style(color="#98c379"),
        "js.arrow": Style(color="#e06c75"),
        "js.module": Style(color="#e06c75"),
        "js.class": Style(color="#c678dd", bold=True),
        "js.decorator": Style(color="#e06c75", italic=True),
        "js.function": Style(color="#61afef"),
        "js.method": Style(color="#61afef"),
        "js.property": Style(color="#61afef"),
        "js.jsx.tag": Style(color="#e06c75"),
        "js.jsx.attribute": Style(color="#61afef"),
        "js.jsx.text": Style(color="#abb2bf"),
        "rust.attribute": Style(color="#e06c75", italic=True),
        "rust.derive": Style(color="#e06c75", italic=True),
        "rust.macro": Style(color="#61afef"),
        "rust.lifetime": Style(color="#e06c75", italic=True),
        "rust.trait": Style(color="#c678dd"),
        "rust.type": Style(color="#c678dd"),
        "rust.self": Style(color="#e06c75", italic=True),
        "go.package": Style(color="#e06c75"),
        "go.builtin": Style(color="#61afef", bold=True),
        "go.type": Style(color="#c678dd"),
        "go.struct": Style(color="#c678dd"),
        "go.interface": Style(color="#c678dd"),
        "java.annotation": Style(color="#e06c75", italic=True),
        "java.class": Style(color="#c678dd", bold=True),
        "java.import": Style(color="#e06c75"),
        "java.package": Style(color="#e06c75"),
        "java.this": Style(color="#e06c75", italic=True),
        "c.include": Style(color="#e06c75"),
        "c.macro": Style(color="#e06c75"),
        "c.struct": Style(color="#c678dd"),
        "c.type": Style(color="#c678dd"),
        "cpp.class": Style(color="#c678dd", bold=True),
        "cpp.namespace": Style(color="#c678dd"),
        "cpp.template": Style(color="#c678dd"),
        "sql.keyword": Style(color="#e06c75"),
        "sql.function": Style(color="#61afef"),
        "sql.operator": Style(color="#e06c75"),
        "sql.table": Style(color="#c678dd"),
        "sql.column": Style(color="#d19a66"),
        "sql.alias": Style(color="#abb2bf", italic=True),
        "sql.string": Style(color="#98c379"),
        "sql.number": Style(color="#d19a66"),
        "sql.comment": Style(color="#5c6370", italic=True),
        "regex.group": Style(color="#d19a66"),
        "regex.quantifier": Style(color="#e06c75", bold=True),
        "regex.boundary": Style(color="#abb2bf"),
        "regex.characterClass": Style(color="#abb2bf"),
        "regex.alternation": Style(color="#e06c75", bold=True),
        "regex.anchor": Style(color="#d29922", bold=True),
        "regex.captureGroup": Style(color="#98c379"),
        "regex.captureGroupName": Style(color="#98c379", italic=True),
        "markdown.inlineCode": Style(color="#98c379"),
        "markdown.codeBlock": Style(color="#98c379"),
        "markdown.codeBlock.info": Style(color="#61afef"),
        "markdown.link": Style(color="#61afef", underline=True),
        "markdown.list": Style(color="#e06c75"),
        "markdown.emphasis": Style(color="#abb2bf", italic=True),
        "markdown.strong": Style(color="#abb2bf", bold=True),
        "markdown.heading": Style(color="#d19a66", bold=True),
        "markdown.quote": Style(color="#5c6370", italic=True),
        "markdown.hr": Style(color="#5c6370"),
        "toml.key": Style(color="#61afef"),
        "toml.boolean": Style(color="#d19a66", bold=True),
        "toml.string": Style(color="#98c379"),
        "toml.number": Style(color="#d19a66"),
        "docker.keyword": Style(color="#e06c75"),
        "docker.instruction": Style(color="#61afef", bold=True),
        "docker.argument": Style(color="#c678dd"),
        "docker.envvar": Style(color="#d19a66"),
    }


)

class FileEditor(TextArea):
    current_file: str = ""
    editing: bool = reactive(False)
    
    EXTENSION_TO_LANGUAGE = {
        ".py": "python",
        ".js": "javascript",
        ".ts": "typescript",
        ".html": "html",
        ".css": "css",
        ".json": "json",
        ".md": "markdown",
        ".c": "c",
        ".cpp": "cpp",
        ".h": "c",
        ".hpp": "cpp",
        ".java": "java",
        ".go": "go",
        ".rs": "rust",
        ".rb": "ruby",
        ".php": "php",
        ".sh": "bash",
        ".yaml": "yaml",
        ".yml": "yaml",
        ".toml": "toml",
        ".xml": "xml",
        ".sql": "sql",
        ".lua": "lua",
        ".dart": "dart",
        ".swift": "swift",
        ".kt": "kotlin",
        ".scala": "scala",
        ".r": "r",
        ".jsx": "javascript",
        ".tsx": "typescript",
    }
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.show_line_numbers = True
        self.tab_behavior = "indent"
        
        self.register_theme(my_theme)
        self.theme = "EditorTheme"
        
        print(f"Available languages: {self.available_languages}")
        print(f"Available themes: {self.available_themes}")

    def set_content(self, new_content, filename):
        self.current_file = filename
        self.load_text(new_content)
        self.read_only = False
        self.editing = True
        self.disabled = False
        
        self.set_language_from_filename(filename)

    def set_language_from_filename(self, filename):
        """Set the appropriate language for syntax highlighting based on file extension"""
        if not filename:
            return
            
        _, ext = os.path.splitext(filename.lower())
        
        print(f"File extension: {ext}")
        
        if ext in self.EXTENSION_TO_LANGUAGE:
            language = self.EXTENSION_TO_LANGUAGE[ext]

            print(f"Selected language: {language}")
            print(f"Available languages: {self.available_languages}")
            
            if language in self.available_languages:
                self.language = language
                self.app.notify(f"Syntax highlighting enabled: {language}")
                self.set_timer(0.5, self.debug_highlights)
            else:
                self.language = None
                self.app.notify(f"Language '{language}' not available for highlighting")
        else:
            self.language = None

    def debug_highlights(self):
        """Debug method to print the highlights being generated"""
        if hasattr(self, "_highlights") and self._highlights:
            self.app.notify(f"Found {len(self._highlights)} highlight groups")
            for line_idx, highlights in enumerate(self._highlights):
                if highlights:
                    print(f"Line {line_idx} highlights: {highlights}")
                    if line_idx > 5:
                        break
        else:
            self.app.notify("No highlights found")

    def save_file(self):
        if self.current_file:
            with open(self.current_file, "w", encoding="utf-8") as f:
                f.write(self.text)
            self.app.notify(f"Saved: {self.current_file}")

    def exit_editing(self):
        self.read_only = True
        self.disabled = True
        self.editing = False
        self.app.active_widget = self.app.directory
        self.app.directory.browsing = True



import keyboard
from textual.app import App, ComposeResult
from textual.containers import Horizontal, HorizontalScroll, Vertical
from textual.widgets import Static
from textual.events import Key
from textual.reactive import reactive
import sys
import os

from .lazydocker_screen import LazyDockerScreen
from .lazygit_screen import LazyGitScreen

from .fileEditor import FileEditor
from .directory import Directory
from .terminal import Terminal

class CommandFooter(Static):
    def on_mount(self):
        self.update("Commands: (Ctrl+q) Quit   (Enter) Create File   (Backspace) Delete File   (Ctrl+s) Save File   (Ctrl+2) Dir Mode    (Ctrl+3) Edit Mode    (Ctrl+5) Terminal   (Ctrl+g) Git mode")

class MyApp(App):
    CSS = """
    Screen {
    layout: vertical;
    background: #0C0C0C;
    }
    Horizontal {
        layout: horizontal;
        height: 1fr;
    }
    Directory {
        width: 25%;
        height: 100%;
    }
    FileEditor {
        height: 100%;
    }
    Terminal {
        height: 30%;
    }
    CommandFooter {
        dock: bottom;
        height: auto;
    }
    """
    
    current_mode = reactive("directory")

    def __init__(self):
        super().__init__()
        self.active_widget = None
        self.cursor_row = 0
        self.cursor_column = 0

    def compose(self) -> ComposeResult:
        self.directory = Directory()
        self.file_editor = FileEditor()
        self.terminal = Terminal()
        self.footer = CommandFooter()

        with Horizontal():
            yield self.directory
            with Vertical():
                with HorizontalScroll():
                    yield self.file_editor
                yield self.terminal
        yield self.footer

        self.active_widget = self.directory

    def on_mount(self):
        self.directory.browsing = True
        self.file_editor.editing = False
        self.terminal.is_active = False

    def on_key(self, event):
    
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("q"):
            self.exit()
            os.system("cls")
            return
        
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("2"):
            self.switch_to_directory_mode()
            return
            
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("3"):
            self.switch_to_editor_mode()
            return
            
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("5"):
            self.switch_to_terminal_mode()
            return
        
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("g"):
            self.push_screen(LazyGitScreen())
            return
        
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("d"):
            self.push_screen(LazyDockerScreen())
            return
        
        if self.current_mode == "directory":
            if keyboard.is_pressed("ctrl") and keyboard.is_pressed("s"):
                return
            if hasattr(self.directory, "on_key"):
                self.directory.on_key(event)
                
        elif self.current_mode == "editor":
            if keyboard.is_pressed("ctrl") and keyboard.is_pressed("s"):
                self.file_editor.save_file()
                return
            if hasattr(self.file_editor, "on_key"):
                self.file_editor.on_key(event)
                
        elif self.current_mode == "terminal":
            if hasattr(self.terminal, "on_key"):
                self.terminal.on_key(event)
    
    def switch_to_directory_mode(self):
        self.current_mode = "directory"
        self.directory.browsing = True
        self.file_editor.editing = False
        self.terminal.is_active = False
        self.active_widget = self.directory
        self.directory.focus()
        self.file_editor.exit_editing()
        self.refresh_ui()
    
    def switch_to_editor_mode(self):
        self.current_mode = "editor"
        self.directory.browsing = False
        self.file_editor.editing = True
        self.terminal.is_active = False
        self.active_widget = self.file_editor
        self.file_editor.focus()
        self.refresh_ui()
    
    def switch_to_terminal_mode(self):
        self.current_mode = "terminal"
        self.directory.browsing = False
        self.file_editor.editing = False
        self.terminal.is_active = True
        self.active_widget = self.terminal
        self.terminal.focus()
        self.file_editor.exit_editing()
        self.refresh_ui()
    
    def refresh_ui(self):
        self.directory.render_files()
        
        if hasattr(self.terminal, "output_buffer"):
            self.terminal.renderable = self.terminal.render()
            self.terminal.refresh(layout=True)
            
        if hasattr(self.file_editor, "refresh"):
            self.file_editor.refresh()

def run():
    MyApp().run()



from textual.widgets import Static
from textual.reactive import reactive
from rich.console import Console, RenderableType
from rich.text import Text
from rich import box
from rich.panel import Panel
from textual import events
import sys
import subprocess
import threading
import queue
import os
import keyboard

class Terminal(Static):
    DEFAULT_CSS = """
    Terminal {
        color: #FFFFFF;
        height: 1fr;
        padding: 0 1;
    }
    """
    
    BINDINGS = [
        ("ctrl+c", "send_ctrl_c", "Send Ctrl+C"),
    ]
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.input_buffer = ""
        self.output_buffer = []
        self.output_queue = queue.Queue()
        self.cursor_position = 0
        self.process = None
        self.can_focus = True
        self.terminal_size = (24, 80) 
        self.visible_lines = 20
        self.prompt = "PS > "
        self.is_active = False
        self._updating = False
    
    def on_mount(self):
        self.terminal_size = (self.size.height, self.size.width)
        self.visible_lines = self.terminal_size[0] - 2
        self.start_shell()
        self.set_interval(0.05, self.update_output)
    
    def on_resize(self, event):
        if hasattr(self, "size"):
            self.terminal_size = (self.size.height, self.size.width)
            self.visible_lines = self.terminal_size[0] - 2
    
    def start_shell(self):
        self.process = subprocess.Popen(
            ["powershell.exe", "-NoLogo"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            creationflags=subprocess.CREATE_NO_WINDOW
        )
        
        threading.Thread(target=self._read_output, args=(self.process.stdout,), daemon=True).start()
        threading.Thread(target=self._read_output, args=(self.process.stderr,), daemon=True).start()
        self.output_buffer.append(f"{self.prompt}")
    
    def _read_output(self, pipe):
        while self.process and self.process.poll() is None:
            try:
                line = pipe.readline()
                if not line:
                    char = pipe.read(1)
                    if not char:
                        break
                    self.output_queue.put(char)
                else:
                    self.output_queue.put(line)
            except (OSError, ValueError):
                break
    
    def update_output(self):
        updated = False
        while not self.output_queue.empty():
            try:
                data = self.output_queue.get_nowait()
                if isinstance(data, bytes):
                    try:
                        text = data.decode('utf-8', errors='replace')
                    except UnicodeDecodeError:
                        text = data.decode('latin-1')
                    self.output_buffer.append(text)
                else:
                    self.output_buffer.append(data)
                updated = True
            except queue.Empty:
                break
        
        if len(self.output_buffer) > 5000:
            self.output_buffer = self.output_buffer[-5000:]
        
        if updated:
            self.renderable = self.render()
            self.refresh(layout=True)
    
    def render(self) -> RenderableType:
        if not hasattr(self, "output_buffer") or self.output_buffer is None:
            return Panel(
                "",
                title="PowerShell",
                border_style="#555555",
                box=box.ROUNDED
            )
            
        content = "".join(self.output_buffer)
        content_lines = content.split("\n")
        visible_content_lines = content_lines[-self.visible_lines:] if content_lines else []
        visible_content = "\n".join(visible_content_lines)
        
        if self.has_focus and self.is_active:
            cursor = "█"
            input_line = (
                self.input_buffer[:self.cursor_position] + 
                cursor + 
                self.input_buffer[self.cursor_position:]
            )
            
            if not visible_content.endswith(self.prompt):
                visible_content += self.prompt
                
            if visible_content.endswith(self.prompt):
                visible_content = visible_content[:-len(self.prompt)] + self.prompt + input_line
        
        terminal_content = Text(visible_content)
        
        border_style = "#007FFF" if self.is_active else "#555555"
        
        return Panel(
            terminal_content,
            title="PowerShell",
            border_style=border_style,
            box=box.ROUNDED
        )
    
    def action_send_ctrl_c(self):
        if not self.is_active:
            return
            
        if self.process and self.process.poll() is None:
            try:
                self.process.send_signal(subprocess.signal.CTRL_C_EVENT)
            except:
                self.write_to_terminal("\x03")
    
    def write_to_terminal(self, data):
        if not self.process or self.process.poll() is not None:
            return
        
        try:
            self.process.stdin.write(data)
            self.process.stdin.flush()
        except (BrokenPipeError, IOError):
            self.output_buffer.append("[Process terminated]\n")
    
    def on_key(self, event: events.Key):
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("q"):
            return
        
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("2"):
            return
            
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("3"):
            return
            
        if keyboard.is_pressed("ctrl") and keyboard.is_pressed("5"):
            return
        
        if not self.has_focus or not self.is_active:
            return
        
        event.prevent_default()
        event.stop()
        
        if event.key == "escape":
            return
        
        if event.key == "enter":
            command = self.input_buffer + "\n"
            
            if self.output_buffer and self.output_buffer[-1].endswith(self.prompt):
                self.output_buffer[-1] = self.output_buffer[-1][:-len(self.prompt)]
            
            self.output_buffer.append(f"{self.prompt}{self.input_buffer}\n")
            
            self.input_buffer = ""
            self.cursor_position = 0
            
            self.write_to_terminal(command)
            
            self.output_buffer.append(f"{self.prompt}")
            
            self.renderable = self.render()
            self.refresh(layout=True)
            
        elif event.key == "backspace":
            if self.cursor_position > 0:
                self.input_buffer = (
                    self.input_buffer[:self.cursor_position - 1] + 
                    self.input_buffer[self.cursor_position:]
                )
                self.cursor_position -= 1
                self.renderable = self.render()
                self.refresh(layout=True)
                
        elif event.key == "delete":
            if self.cursor_position < len(self.input_buffer):
                self.input_buffer = (
                    self.input_buffer[:self.cursor_position] + 
                    self.input_buffer[self.cursor_position + 1:]
                )
                self.renderable = self.render()
                self.refresh(layout=True)
                
        elif event.key == "left":
            if self.cursor_position > 0:
                self.cursor_position -= 1
                self.renderable = self.render()
                self.refresh(layout=True)
                
        elif event.key == "right":
            if self.cursor_position < len(self.input_buffer):
                self.cursor_position += 1
                self.renderable = self.render()
                self.refresh(layout=True)
                
        elif event.key == "home":
            self.cursor_position = 0
            self.renderable = self.render()
            self.refresh(layout=True)
            
        elif event.key == "end":
            self.cursor_position = len(self.input_buffer)
            self.renderable = self.render()
            self.refresh(layout=True)
            
        elif event.key == "tab":
            self.write_to_terminal("\t")
            self.renderable = self.render()
            self.refresh(layout=True)
            
        elif event.key == "ctrl+l":
            self.output_buffer = [f"{self.prompt}"]
            self.renderable = self.render()
            self.refresh(layout=True)
            
        elif event.is_printable:
            self.input_buffer = (
                self.input_buffer[:self.cursor_position] + 
                event.character + 
                self.input_buffer[self.cursor_position:]
            )
            self.cursor_position += 1
            self.renderable = self.render()
            self.refresh(layout=True)
    
    def on_focus(self) -> None:
        self.renderable = self.render()
        self.refresh(layout=True)
    
    def on_blur(self) -> None:
        self.renderable = self.render()
        self.refresh(layout=True)
        
    def on_unmount(self) -> None:
        if self.process and self.process.poll() is None:
            self.process.terminate()
            try:
                self.process.wait(timeout=1)
            except subprocess.TimeoutExpired:
                self.process.kill()
                
    def refresh(self, **kwargs):
        super().refresh(**kwargs)



from textual.widgets import Static, Input, Label, Button
from textual.reactive import reactive
from textual.containers import Container, Horizontal
from rich.panel import Panel
from rich.text import Text
import os
import shutil


class DeleteConfirmDialog(Container):
    def __init__(self, file_path):
        super().__init__()
        self.file_path = file_path
        
    def compose(self):
        file_name = os.path.basename(self.file_path)
        yield Label(f"Delete '{file_name}'?")
        
        with Horizontal(id="button_container"):
            yield Button("Yes", id="yes_button", variant="primary")
            yield Button("No", id="no_button")
        
    def on_mount(self):
        self.query_one("#yes_button").focus()
        
    def on_button_pressed(self, event):
        if event.button.id == "yes_button":
            try:
                if os.path.isdir(self.file_path):
                    shutil.rmtree(self.file_path)
                else:
                    os.remove(self.file_path)
                self.app.directory.update_directory()
                self.app.notify(f"Deleted: {os.path.basename(self.file_path)}")
            except Exception as e:
                self.app.notify(f"Error deleting file: {str(e)}", severity="error")
        
        self.app.directory.browsing = True
        self.remove()
    
    def on_key(self, event):
        if event.key == "left":
            self.query_one("#yes_button").focus()
        elif event.key == "right":
            self.query_one("#no_button").focus()
        elif event.key == "escape":
            self.app.directory.browsing = True
            self.remove()


class FileNameDialog(Container):
    def __init__(self, directory_path):
        super().__init__()
        self.directory_path = directory_path
        
    def compose(self):
        yield Label("Enter file name:")
        yield Input(id="filename_input")
        
    def on_mount(self):
        self.query_one(Input).focus()
        
    def on_input_submitted(self, event):
        filename = event.value.strip()
        if filename:
            file_path = os.path.join(self.directory_path, filename)
            
            try:
                with open(file_path, "w") as f:
                    pass
                
                self.app.directory.update_directory()
                self.app.directory.browsing = True
                self.remove()
                
                if os.path.isfile(file_path):
                    self.app.file_editor.set_content("", file_path)
            except Exception as e:
                self.app.notify(f"Error creating file: {str(e)}", severity="error")
                self.remove()
    
    def on_key(self, event):
        if event.key == "escape":
            self.app.directory.browsing = True
            self.remove()


class Directory(Static):
    selected_index: int = reactive(0)
    files: list = []
    browsing: bool = reactive(True)
    expanded_folders: set = set()
    scroll_offset: int = reactive(0)

    def on_mount(self):
        self.update_directory()

    def update_directory(self):
        self.files = os.listdir(".")
        self.render_files()

    def get_nested_files(self, folder_path, current_indent):
        nested_items = []
        try:
            subfiles = [os.path.join(folder_path, f) for f in os.listdir(folder_path)]
            for subfile in sorted(subfiles):
                nested_items.append((subfile, current_indent))
                if os.path.isdir(subfile) and subfile in self.expanded_folders:
                    nested_items.extend(self.get_nested_files(subfile, current_indent + 1))
        except (PermissionError, OSError):
            pass
        return nested_items

    def render_files(self):
        display_items = []
        
        for file_path in sorted(self.files):
            display_items.append((file_path, 0))
            
            if os.path.isdir(file_path) and file_path in self.expanded_folders:
                display_items.extend(self.get_nested_files(file_path, 1))
        
        self.display_items = display_items
        
        visible_height = self.size.height - 2 
        if visible_height < 1:
            visible_height = 26
        
        if self.selected_index < self.scroll_offset:
            self.scroll_offset = self.selected_index
        elif self.selected_index >= self.scroll_offset + visible_height:
            self.scroll_offset = self.selected_index - visible_height + 1
    
        max_scroll = max(0, len(display_items) - visible_height)
        self.scroll_offset = max(0, min(self.scroll_offset, max_scroll))
        
        visible_items = display_items[self.scroll_offset:self.scroll_offset + visible_height]
        
        file_list_items = []
        for i, (file_path, indent_level) in enumerate(visible_items):
            actual_index = i + self.scroll_offset
            prefix = "    " * indent_level
            file_name = os.path.basename(file_path)
            
            if os.path.isdir(file_path):
                if file_path in self.expanded_folders:
                    icon = "▼ "
                else:
                    icon = "▶ "
            else:
                icon = "  "
                
            display_text = f"{prefix}{icon}{file_name}"
            
            if actual_index == self.selected_index:
                file_list_items.append(f"[green]{display_text}[/green]")
            else:
                file_list_items.append(display_text)
        
        title = "Directory"
        if self.scroll_offset > 0:
            title = "↑ " + title
        if self.scroll_offset + visible_height < len(display_items):
            title = title + " ↓"
            
        file_list = "\n".join(file_list_items)
        
        border_style = "#007FFF" if self.browsing and self.app.current_mode == "directory" else "#555555"
        
        self.update(Panel(Text.from_markup(file_list), title=title, border_style=border_style))

    def on_key(self, event):
        if self.app.current_mode != "directory" or not self.browsing:
            return
        
        event.prevent_default()
        event.stop()
        
        if event.key == "down" and self.selected_index < len(self.display_items) - 1:
            self.selected_index += 1
            self.render_files()
        elif event.key == "up" and self.selected_index > 0:
            self.selected_index -= 1
            self.render_files()
        elif event.key == "enter":
            if self.selected_index < len(self.display_items):
                selected_path, _ = self.display_items[self.selected_index]
                
                if os.path.isdir(selected_path):
                    directory_path = selected_path
                else:
                    directory_path = os.path.dirname(selected_path)
                    if not directory_path:
                        directory_path = "."
                
                self.browsing = False
                dialog = FileNameDialog(directory_path)
                self.app.mount(dialog)
            else:
                self.browsing = False
                dialog = FileNameDialog(".")
                self.app.mount(dialog)
        elif event.key == "backspace" or event.key == "delete":
            if self.selected_index < len(self.display_items):
                selected_path, _ = self.display_items[self.selected_index]
                self.browsing = False
                dialog = DeleteConfirmDialog(selected_path)
                self.app.mount(dialog)
                
        elif event.key == "space":
            if self.selected_index < len(self.display_items):
                selected_path, _ = self.display_items[self.selected_index]
                
                if os.path.isdir(selected_path):
                    if selected_path in self.expanded_folders:
                        self.expanded_folders.remove(selected_path)
                    else:
                        self.expanded_folders.add(selected_path)
                    self.render_files()
                elif os.path.isfile(selected_path):
                    try:
                        with open(selected_path, "r", encoding="utf-8", errors="ignore") as f:
                            file_content = f.read()
                        self.app.file_editor.set_content(file_content, selected_path)
                    except Exception as e:
                        self.app.notify(f"Error opening file: {str(e)}", severity="error")
